"""
See http://docs.djangoproject.com/en/dev/ref/templates/api/#using-an-alternative-template-language

"""

from django.template.loaders import app_directories
from django.conf import settings
from django.template import TemplateDoesNotExist
from jinja2 import nodes
from djinja.template import Environment
import jinja2


app_template_dirs = app_directories.app_template_dirs + settings.TEMPLATE_DIRS


from djinja.template import Template
from djinja.template.defaultfunctions import url

class Loader(app_directories.Loader):
    is_usable = True
    env = Environment(loader=jinja2.FileSystemLoader(app_template_dirs),extensions=['jinja2.ext.i18n'])
    
    def load_template(self, template_name, template_dirs=None):
        try:
            template = self.env.get_template(template_name)
            return template, template.filename
        except jinja2.TemplateNotFound:
            raise TemplateDoesNotExist(template_name)



def make_jinja2_tag(node,*args,**kwargs):
    def f(*args,**kwargs):
        return node(*args,**kwargs).render({})
    return f



"""Jinja2-ports of many of Django's default filters.

TODO: Most of the filters in here need to be updated for autoescaping.

TAKED FROM COFFIN: https://github.com/cdleary/coffin
(and adapted for djinja library)
"""

from djinja.template.base import Library
from jinja2.runtime import Undefined
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode

register = Library()

@register.filter
def url(view_name, *args, **kwargs):
    from coffin.template.defaulttags import url
    return url._reverse(view_name, args, kwargs)
    
@register.filter
def timesince(value, *arg):
    if value is None or isinstance(value, Undefined):
        return u''
    from django.utils.timesince import timesince
    return timesince(value, *arg)

@register.filter
def date(value, arg=None):
    if value is None or isinstance(value, Undefined):
        return u''
    from django.conf import settings
    from django.utils.dateformat import format
    if arg is None:
        arg = settings.DATETIME_FORMAT
    return format(value, arg)

@register.filter
def time(value, arg=None):
    if value is None or isinstance(value, Undefined):
        return u''
    from django.conf import settings
    from django.utils.dateformat import time_format
    if arg is None:
        arg = settings.TIME_FORMAT
    return time_format(value, arg)

@register.filter
def truncatewords(value, length):
    # Jinja2 has it's own ``truncate`` filter that supports word
    # boundaries and more stuff, but cannot deal with HTML.
    from django.utils.text import truncate_words
    return truncate_words(value, int(length))

@register.filter
def truncatewords_html(value, length):
    from django.utils.text import truncate_html_words
    return truncate_html_words(value, int(length))

@register.filter
def pluralize(value, s1='s', s2=None):
    """Like Django's pluralize-filter, but instead of using an optional
    comma to separate singular and plural suffixes, it uses two distinct
    parameters.

    It also is less forgiving if applied to values that do not allow
    making a decision between singular and plural.
    """
    if s2 is not None:
        singular_suffix, plural_suffix = s1, s2
    else:
        plural_suffix = s1
        singular_suffix = ''

    try:
        if int(value) != 1:
            return plural_suffix
    except TypeError: # not a string or a number; maybe it's a list?
        if len(value) != 1:
            return plural_suffix
    return singular_suffix

@register.filter
def floatformat(value, arg=-1):
    """Builds on top of Django's own version, but adds strict error
    checking, staying with the philosophy.
    """
    from django.template.defaultfilters import floatformat
    from coffin.interop import django_filter_to_jinja2
    arg = int(arg)  # raise exception
    result = django_filter_to_jinja2(floatformat)(value, arg)
    if result == '':  # django couldn't handle the value
        raise ValueError(value)
    return result
    
from djinja.template.base import import_library
django_filters = import_library('django.template.defaultfilters').filters
for filter in django_filters.keys():
    register.filter(django_filters[filter])


from djinja import template
from django.utils.encoding import iri_to_uri


register = template.Library()

@register.tag #You can use this form
def basic_tag():
    return 'Basic tag!'

def reverse(a): 
    return a[::-1]
    
register.filter(reverse) #Or this! As you can see, you can set the same function as tag or filter :D
register.tag(reverse)

register.set(var_example='Example variable')


from django.contrib.admin.templatetags.admin_modify import submit_row,prepopulated_fields_js
from djinja import template
import jinja2 

register = template.Library()
register.inclusion_tag('admin/submit_line.html', submit_row,True)
register.inclusion_tag('admin/prepopulated_fields_js.html', prepopulated_fields_js,True)


from djinja import template
from django.core.urlresolvers import resolve

import jinja2
register = template.Library()
@jinja2.contextfunction
def active (context,*args):
    match = resolve(context['request_path'])
    return (match.url_name in args)
    #if context['request_path'] in ( reverse(url) for url in args ):
    #    return True
    #return False
    
register.tag(active)



from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    url(r'^$', 'website_example.views.index', name='index'),
    url(r'^haml$', 'website_example.views.index_haml', name='index_haml'),

    # url(r'^$', 'website_example.views.home', name='home'),
    # url(r'^website_example/', include('website_example.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    # url(r'^admin/', include(admin.site.urls)),
)



from setuptools import setup, find_packages

setup(
    name='Djinja',
    version=".".join(map(str, __import__("djinja").__version__)),
    description='Easy Jinja2 templates integration in Django (including admin), with HAML syntax support.',
    long_description=open('README.rst').read(),
    author='Syrus Akbary Nieto',
    author_email='dimension.net@gmail.com',
    url='http://github.com/syrusakbary/djinja',
    license='BSD',
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    zip_safe=False, # because we're including media that Django needs
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)